local CBFP_oldUEL0001 = UEL0001

UEL0001 = Class(CBFP_oldUEL0001) {    

    NotifyOfPodDeath = function(self, pod)

#### Brute51 Pod Fix part 1/2 ---------------------------------
# In this fix I'm seperating 'Left pod' from the left should pod (first upgrade) and the 'Right pod' from the 
# right shoulder pod (second upgrade). This way if a pod is destroyed I dont have to reset the other pod so now it
# can continue to do what it was doing.
# When a pod is destroyed this removes either the 2nd upgrade if we had 2 pods, or the first if we had 1 pods.
####
        if pod == 'LeftPod' then
            if self.HasRightPod then
                TWalkingLandUnit.CreateEnhancement(self, 'RightPodRemove') # cant use CreateEnhancement function
                TWalkingLandUnit.CreateEnhancement(self, 'LeftPod') #makes the correct upgrade icon light up
                if self.LeftPod and not self.LeftPod:IsDead() then
                    self.LeftPod:Kill()
                end
            else
                self:CreateEnhancement('LeftPodRemove')
            end
            self.HasLeftPod = false

        elseif pod == 'RightPod' then   #basically the same as above but we can use the CreateEnhancement function this time
            if self.HasLeftPod then
                TWalkingLandUnit.CreateEnhancement(self, 'RightPodRemove') # cant use CreateEnhancement function
                TWalkingLandUnit.CreateEnhancement(self, 'LeftPod') #makes the correct upgrade icon light up
                if self.LeftPod and not self.LeftPod:IsDead() then
                    self.RightPod:Kill()
                end
            else
                self:CreateEnhancement('LeftPodRemove')
            end
            self.HasRightPod = false

        end
        self:RequestRefreshUI()

# end Pod Fix 1/2 ---------------------------------------
    end,

    CreateEnhancement = function(self, enh)
        local bp = self:GetBlueprint().Enhancements[enh]
        if not bp then return end


#### Brute51 Pod Fix part 2/2 ---------------------------------
# Here we're adding pods when a pod upgrade is done. The pod is added to either Leftpod or Rightpod, depending on
# which variable isnt used yet. This means it could happen Leftpod is used for the 2nd pod.
####

        if enh == 'LeftPod' or enh == 'RightPod' then

            TWalkingLandUnit.CreateEnhancement(self, enh) # moved from top to here so this happens only once for each enhancement

            # making sure we have up to date information (dont delete! needed for bug fix below)
            if not self.LeftPod or self.LeftPod:IsDead() then
                self.HasLeftPod = false
            end
            if not self.RightPod or self.RightPod:IsDead() then
                self.HasRightPod = false
            end

            # fix for a bug that occurs when pod 1 is destroyed while upgrading to get pod 2
            if enh == 'RightPod' and (not self.HasLeftPod or not self.HasRightPod) then
                TWalkingLandUnit.CreateEnhancement(self, 'RightPodRemove')
                TWalkingLandUnit.CreateEnhancement(self, 'LeftPod')
            end

            # add new pod to left or right
            if not self.HasLeftPod then
                local location = self:GetPosition('AttachSpecial02')
                local pod = CreateUnitHPR('UEA0001', self:GetArmy(), location[1], location[2], location[3], 0, 0, 0)
                pod:SetCreator(self)
                pod:SetParent(self, 'LeftPod')
                self.Trash:Add(pod)
                self.LeftPod = pod
                self.HasLeftPod = true
            else
                local location = self:GetPosition('AttachSpecial01')
                local pod = CreateUnitHPR('UEA0001', self:GetArmy(), location[1], location[2], location[3], 0, 0, 0)
                pod:SetCreator(self)
                pod:SetParent(self, 'RightPod')
                self.Trash:Add(pod)
                self.RightPod = pod
                self.HasRightPod = true
            end

            # highlight correct icons: right if we have 2 pods, left if we have 1 pod (no other possibilities)
            if self.HasLeftPod and self.HasRightPod then
                TWalkingLandUnit.CreateEnhancement(self, 'RightPod')
            else
                TWalkingLandUnit.CreateEnhancement(self, 'LeftPod')
            end

        # for removing the pod upgrades
        elseif enh == 'RightPodRemove' then
            TWalkingLandUnit.CreateEnhancement(self, enh) # moved from top to here so this happens only once for each enhancement
            if self.RightPod and not self.RightPod:IsDead() then
                self.RightPod:Kill()
                self.HasRightPod = false
            end
            if self.LeftPod and not self.LeftPod:IsDead() then
                self.LeftPod:Kill()
                self.HasLeftPod = false
            end
        elseif enh == 'LeftPodRemove' then
            TWalkingLandUnit.CreateEnhancement(self, enh) # moved from top to here so this happens only once for each enhancement
            if self.LeftPod and not self.LeftPod:IsDead() then
                self.LeftPod:Kill()
                self.HasLeftPod = false
            end

# end Pod Fix 2/2 ---------------------------------------

        elseif enh =='DamageStablization' then
            TWalkingLandUnit.CreateEnhancement(self, enh) # moved from top to here so this happens only once for each enhancement
            # added by brute51 - fix for bug SCU regen upgrade doesnt stack with veteran bonus [140]
            local bpRegenRate = bp.NewRegenRate or 0
            if not Buffs['UefACURegenerateBonus'] then
               BuffBlueprint {
                    Name = 'UefACURegenerateBonus',
                    DisplayName = 'UefACURegenerateBonus',
                    BuffType = 'ACUREGENERATEBONUS',
                    Stacks = 'ALWAYS',
                    Duration = -1,
                    Affects = {
                        Regen = {
                            Add = bpRegenRate,
                            Mult = 1.0,
                        },
                    },
                } 
            end
            if Buff.HasBuff( self, 'UefACURegenerateBonus' ) then
                Buff.RemoveBuff( self, 'UefACURegenerateBonus' )
            end  
            Buff.ApplyBuff(self, 'UefACURegenerateBonus')
        elseif enh =='DamageStablizationRemove' then
            TWalkingLandUnit.CreateEnhancement(self, enh) # moved from top to here so this happens only once for each enhancement
            # added by brute51 - fix for bug SCU regen upgrade doesnt stack with veteran bonus [140]
            if Buff.HasBuff( self, 'UefACURegenerateBonus' ) then
                Buff.RemoveBuff( self, 'UefACURegenerateBonus' )
            end  

        else
            CBFP_oldUEL0001.CreateEnhancement(self, enh)
        end
    end,
}

TypeClass = UEL0001